@@ -181,7 +181,7 @@ giop_send_buffer_write(GIOPSendBuffer *send_buffer)
   nvecs = GIOP_MESSAGE_BUFFER(send_buffer)->iovecs->len;
   curvec = (struct iovec *)GIOP_MESSAGE_BUFFER(send_buffer)->iovecs->data;
 
-#if defined(DEBUG) && 0
+#if defined(ORBIT_DEBUG) && 0
   g_print("Message of length %d looks like:\n",
 	  GIOP_MESSAGE_BUFFER(send_buffer)->message_header.message_size);
   for(sum = 0, i = 2; i < nvecs; i++) {
@@ -246,7 +246,7 @@ giop_send_buffer_use(GIOPConnection *connection)
 
   g_mem_chunk_reset(retval->indirects);
   retval->indirect = g_chunk_new(gpointer, retval->indirects);
-#ifdef DEBUG
+#ifdef ORBIT_DEBUG
   memset(retval->indirect, '\xFE', GIOP_INDIRECT_CHUNK_SIZE);
 #endif
   retval->indirect_used = 0;
@@ -264,6 +264,8 @@ giop_send_reply_buffer_use(GIOPConnection *connection,
 
   send_buffer = giop_send_buffer_use(connection);
 
+  g_return_val_if_fail(send_buffer, NULL);
+
   GIOP_MESSAGE_BUFFER(send_buffer)->message_header.message_type = GIOP_REPLY;
 
   giop_message_buffer_do_alignment(GIOP_MESSAGE_BUFFER(send_buffer),
@@ -311,6 +313,8 @@ giop_send_locate_reply_buffer_use(GIOPConnection *connection,
 
 	send_buffer = giop_send_buffer_use(connection);
 
+	g_return_val_if_fail(send_buffer, NULL);
+
 	GIOP_MESSAGE_BUFFER(send_buffer)->message_header.message_type = GIOP_LOCATEREPLY;
 
 	APIA(&request_id, sizeof(request_id));
@@ -351,6 +355,8 @@ giop_send_request_buffer_use(GIOPConnection *connection,
 
   send_buffer = giop_send_buffer_use(connection);
 
+  g_return_val_if_fail(send_buffer, NULL);
+
   GIOP_MESSAGE_BUFFER(send_buffer)->message_header.message_type = GIOP_REQUEST;
 
   giop_message_buffer_do_alignment(GIOP_MESSAGE_BUFFER(send_buffer),
@@ -437,6 +443,8 @@ giop_send_locate_request_buffer_use(GIOPConnection *connection,
 
 	send_buffer = giop_send_buffer_use(connection);
 
+	g_return_val_if_fail(send_buffer, NULL);
+
 	GIOP_MESSAGE_BUFFER(send_buffer)->message_header.message_type = GIOP_LOCATEREQUEST;
 
 	APIA(&request_id, sizeof(request_id));
@@ -654,7 +662,7 @@ _giop_send_buffer_append_mem_indirect(GIOPSendBuffer *send_buffer,
 	  send_buffer->indirect_used = 0;
 	  send_buffer->indirect = g_chunk_new(gpointer,
 					      send_buffer->indirects);
-#ifdef DEBUG
+#ifdef ORBIT_DEBUG
 	  memset(send_buffer->indirect, '\xFE', GIOP_INDIRECT_CHUNK_SIZE);
 #endif
 	}
