@@ -177,7 +177,11 @@ ORBit_marshal_value(GIOPSendBuffer *buf,
 	   marshal the discriminator
 	   find out which value we want to use */
 	{
-	    CORBA_TypeCode utc = ORBit_get_union_tag(tc, val);
+	    CORBA_TypeCode utc;
+
+	    *val = ALIGN_ADDRESS(*val, ALIGNOF_CORBA_STRUCT);
+
+	    utc = ORBit_get_union_tag(tc, val);
 
 	    *val = ALIGN_ADDRESS(*val, ALIGNOF_CORBA_STRUCT);
 
@@ -263,31 +267,36 @@ ORBit_marshal_value(GIOPSendBuffer *buf,
     }
 }
 
-static glong ORBit_get_union_switch(CORBA_TypeCode tc, gpointer val)
+static glong ORBit_get_union_switch(CORBA_TypeCode tc, gpointer *val, gboolean update)
 {
+    glong retval;
+
     switch(tc->kind) {
     case CORBA_tk_ulong:
     case CORBA_tk_long:
     case CORBA_tk_enum:
-	return *(CORBA_long *)val;
+	retval = *(CORBA_long *)val;
+	if(update) val += sizeof(CORBA_long);
 	break;
     case CORBA_tk_ushort:
     case CORBA_tk_short:
-	return *(CORBA_short *)val;
+	retval = *(CORBA_short *)val;
+	if(update) val += sizeof(CORBA_short);
 	break;
     case CORBA_tk_char:
     case CORBA_tk_boolean:
     case CORBA_tk_octet:
-	return *(CORBA_octet *)val;
+	retval = *(CORBA_octet *)val;
+	if(update) val += sizeof(CORBA_char);
 	break;
     case CORBA_tk_alias:
-	return ORBit_get_union_switch(tc->subtypes[0], val);
+	return ORBit_get_union_switch(tc->subtypes[0], val, update);
 	break;
     default:
 	g_error("Wow, some nut has passed us a weird type[%d] as a union discriminator!", tc->kind);
     }
 
-    return 0;
+    return retval;
 }
 
 /* This function (and the one above it) exist for the
@@ -298,20 +307,20 @@ static glong ORBit_get_union_switch(CORBA_TypeCode tc, gpointer val)
 */
 CORBA_TypeCode
 ORBit_get_union_tag(CORBA_TypeCode union_tc,
-		    gpointer val)
+		    gpointer *val)
 {
     glong discrim_val, case_val;
     int i;
     CORBA_TypeCode retval = CORBA_OBJECT_NIL;
 
-    discrim_val = ORBit_get_union_switch(union_tc->discriminator, val);
+    discrim_val = ORBit_get_union_switch(union_tc->discriminator, *val, TRUE);
 
     for(i = 0; i < union_tc->sub_parts; i++) {
 	if(i == union_tc->default_index)
 	    continue;
 
 	case_val = ORBit_get_union_switch(union_tc->sublabels[i]._type,
-					  union_tc->sublabels[i]._value);
+					  &union_tc->sublabels[i]._value, FALSE);
 	if(case_val == discrim_val) {
 	    retval = union_tc->subtypes[i];
 	    break;
