@@ -28,10 +28,9 @@
 
 
 /* moved from sequences.h */
+#include "corba_basic_sequences_type.h"
 
-typedef struct CORBA_sequence_octet_struct CORBA_sequence_octet;
-
-typedef struct CORBA_sequence_octet CORBA_ReferenceData;
+typedef CORBA_sequence_octet CORBA_ReferenceData;
 
 #define _CORBA_sequence_ServiceOption_defined
 typedef struct CORBA_sequence_ServiceOption CORBA_sequence_ServiceOption;
