@@ -15,7 +15,7 @@ main (int argc, char *argv[])
 {
     CORBA_Environment ev;
     CORBA_ORB orb;
-    CORBA_long rv;
+    CORBA_double rv;
     char buf[30];
     int i;
 
@@ -66,10 +66,8 @@ main (int argc, char *argv[])
 	      printf("we got exception %d from release!\n", ev._major);
 	      return 1;
 	    }
-#if 0
 	     else
-	      g_message("[client] %d", rv);
-#endif
+	      g_message("[client] %g", rv);
 
 	    echo_client = bec; bec = CORBA_OBJECT_NIL;
     }
