@@ -72,12 +72,16 @@ do_print(PortableServer_Servant servant,
   g_message("[server] %d: %s", any_struct->long_value, 
 	    any_struct->string_value);
 
-
   retval = CORBA_any_alloc();
+#if 0
   retval->_type = (CORBA_TypeCode)TC_TestAnyStruct;
   retval->_value = TestAnyStruct__alloc();
   ((TestAnyStruct *)retval->_value)->long_value = 84;
   ((TestAnyStruct *)retval->_value)->string_value = CORBA_string_dup("Hi there");
+#else
+  retval->_type = (CORBA_TypeCode)TC_null;
+  retval->_value = NULL;
+#endif
 
   CORBA_any_set_release(retval, CORBA_TRUE);
 
